@@ -1,14 +1,11 @@
 import asyncio
+import ssl
 
 try:
     import aiohttp
 except ImportError:  # pragma: no cover
     aiohttp = None
 import six
-try:
-    import websockets
-except ImportError:  # pragma: no cover
-    websockets = None
 
 from . import client
 from . import exceptions
@@ -32,6 +29,10 @@ class AsyncClient(client.Client):
                  versions.
     :param request_timeout: A timeout in seconds for requests. The default is
                             5 seconds.
+    :param ssl_verify: ``True`` to verify SSL certificates, or ``False`` to
+                       skip SSL certificate verification, allowing
+                       connections to servers with self signed certificates.
+                       The default is ``True``.
     """
     def is_asyncio_based(self):
         return True
@@ -180,7 +181,7 @@ class AsyncClient(client.Client):
             self._reset()
             raise exceptions.ConnectionError(
                 'Connection refused by the server')
-        if r.status != 200:
+        if r.status < 200 or r.status >= 300:
             raise exceptions.ConnectionError(
                 'Unexpected status code {} in server response'.format(
                     r.status))
@@ -222,8 +223,8 @@ class AsyncClient(client.Client):
 
     async def _connect_websocket(self, url, headers, engineio_path):
         """Establish or upgrade to a WebSocket connection with the server."""
-        if websockets is None:  # pragma: no cover
-            self.logger.error('websockets package not installed')
+        if aiohttp is None:  # pragma: no cover
+            self.logger.error('aiohttp package not installed')
             return False
         websocket_url = self._get_engineio_url(url, engineio_path,
                                                'websocket')
@@ -238,22 +239,20 @@ class AsyncClient(client.Client):
             self.logger.info(
                 'Attempting WebSocket connection to ' + websocket_url)
 
-        # get the cookies from the long-polling connection so that they can
-        # also be sent the the WebSocket route
-        cookies = None
-        if self.http:
-            cookies = '; '.join(["{}={}".format(cookie.key, cookie.value)
-                                 for cookie in self.http._cookie_jar])
-            headers = headers.copy()
-            headers['Cookie'] = cookies
-
         try:
-            ws = await websockets.connect(
-                websocket_url + self._get_url_timestamp(),
-                extra_headers=headers)
-        except (websockets.exceptions.InvalidURI,
-                websockets.exceptions.InvalidHandshake,
-                OSError):
+            if not self.ssl_verify:
+                ssl_context = ssl.create_default_context()
+                ssl_context.check_hostname = False
+                ssl_context.verify_mode = ssl.CERT_NONE
+                ws = await self.http.ws_connect(
+                    websocket_url + self._get_url_timestamp(),
+                    headers=headers, ssl=ssl_context)
+            else:
+                ws = await self.http.ws_connect(
+                    websocket_url + self._get_url_timestamp(),
+                    headers=headers)
+        except (aiohttp.client_exceptions.WSServerHandshakeError,
+                aiohttp.client_exceptions.ServerConnectionError):
             if upgrade:
                 self.logger.warning(
                     'WebSocket upgrade failed: connection error')
@@ -264,14 +263,14 @@ class AsyncClient(client.Client):
             p = packet.Packet(packet.PING, data='probe').encode(
                 always_bytes=False)
             try:
-                await ws.send(p)
+                await ws.send_str(p)
             except Exception as e:  # pragma: no cover
                 self.logger.warning(
                     'WebSocket upgrade failed: unexpected send exception: %s',
                     str(e))
                 return False
             try:
-                p = await ws.recv()
+                p = (await ws.receive()).data
             except Exception as e:  # pragma: no cover
                 self.logger.warning(
                     'WebSocket upgrade failed: unexpected recv exception: %s',
@@ -284,19 +283,17 @@ class AsyncClient(client.Client):
                 return False
             p = packet.Packet(packet.UPGRADE).encode(always_bytes=False)
             try:
-                await ws.send(p)
+                await ws.send_str(p)
             except Exception as e:  # pragma: no cover
                 self.logger.warning(
                     'WebSocket upgrade failed: unexpected send exception: %s',
                     str(e))
                 return False
             self.current_transport = 'websocket'
-            if self.http:  # pragma: no cover
-                await self.http.close()
             self.logger.info('WebSocket upgrade was successful')
         else:
             try:
-                p = await ws.recv()
+                p = (await ws.receive()).data
             except Exception as e:  # pragma: no cover
                 raise exceptions.ConnectionError(
                     'Unexpected recv exception: ' + str(e))
@@ -357,10 +354,17 @@ class AsyncClient(client.Client):
         if self.http is None or self.http.closed:
             self.http = aiohttp.ClientSession()
         http_method = getattr(self.http, method.lower())
+
         try:
-            return await http_method(
-                url, headers=headers, data=body,
-                timeout=aiohttp.ClientTimeout(total=timeout))
+            if not self.ssl_verify:
+                return await http_method(
+                    url, headers=headers, data=body,
+                    timeout=aiohttp.ClientTimeout(total=timeout), ssl=False)
+            else:
+                return await http_method(
+                    url, headers=headers, data=body,
+                    timeout=aiohttp.ClientTimeout(total=timeout))
+
         except (aiohttp.ClientError, asyncio.TimeoutError) as exc:
             self.logger.info('HTTP %s request to %s failed with error %s.',
                              method, url, exc)
@@ -407,7 +411,8 @@ class AsyncClient(client.Client):
         interval.
         """
         self.pong_received = True
-        self.ping_loop_event.clear()
+        if self.ping_loop_event is None:
+            self.ping_loop_event = self.create_event()
         while self.state == 'connected':
             if not self.pong_received:
                 self.logger.info(
@@ -439,7 +444,7 @@ class AsyncClient(client.Client):
                     'Connection refused by the server, aborting')
                 await self.queue.put(None)
                 break
-            if r.status != 200:
+            if r.status < 200 or r.status >= 300:
                 self.logger.warning('Unexpected status code %s in server '
                                     'response, aborting', r.status)
                 await self.queue.put(None)
@@ -457,7 +462,8 @@ class AsyncClient(client.Client):
         self.logger.info('Waiting for write loop task to end')
         await self.write_loop_task
         self.logger.info('Waiting for ping loop task to end')
-        self.ping_loop_event.set()
+        if self.ping_loop_event:  # pragma: no cover
+            self.ping_loop_event.set()
         await self.ping_loop_task
         if self.state == 'connected':
             await self._trigger_event('disconnect', run_async=False)
@@ -473,8 +479,8 @@ class AsyncClient(client.Client):
         while self.state == 'connected':
             p = None
             try:
-                p = await self.ws.recv()
-            except websockets.exceptions.ConnectionClosed:
+                p = (await self.ws.receive()).data
+            except aiohttp.client_exceptions.ServerDisconnectedError:
                 self.logger.info(
                     'Read loop: WebSocket connection was closed, aborting')
                 await self.queue.put(None)
@@ -492,7 +498,8 @@ class AsyncClient(client.Client):
         self.logger.info('Waiting for write loop task to end')
         await self.write_loop_task
         self.logger.info('Waiting for ping loop task to end')
-        self.ping_loop_event.set()
+        if self.ping_loop_event:  # pragma: no cover
+            self.ping_loop_event.set()
         await self.ping_loop_task
         if self.state == 'connected':
             await self._trigger_event('disconnect', run_async=False)
@@ -547,7 +554,7 @@ class AsyncClient(client.Client):
                     self.logger.warning(
                         'Connection refused by the server, aborting')
                     break
-                if r.status != 200:
+                if r.status < 200 or r.status >= 300:
                     self.logger.warning('Unexpected status code %s in server '
                                         'response, aborting', r.status)
                     self._reset()
@@ -556,9 +563,14 @@ class AsyncClient(client.Client):
                 # websocket
                 try:
                     for pkt in packets:
-                        await self.ws.send(pkt.encode(always_bytes=False))
+                        if pkt.binary:
+                            await self.ws.send_bytes(pkt.encode(
+                                always_bytes=False))
+                        else:
+                            await self.ws.send_str(pkt.encode(
+                                always_bytes=False))
                         self.queue.task_done()
-                except websockets.exceptions.ConnectionClosed:
+                except aiohttp.client_exceptions.ServerDisconnectedError:
                     self.logger.info(
                         'Write loop: WebSocket connection was closed, '
                         'aborting')
