@@ -4,6 +4,7 @@ try:
 except ImportError:  # pragma: no cover
     import Queue as queue
 import signal
+import ssl
 import threading
 import time
 
@@ -38,7 +39,11 @@ def signal_handler(sig, frame):
             client.start_background_task(client.disconnect, abort=True)
         else:
             client.disconnect(abort=True)
-    return original_signal_handler(sig, frame)
+    if callable(original_signal_handler):
+        return original_signal_handler(sig, frame)
+    else:  # pragma: no cover
+        # Handle case where no original SIGINT handler was present.
+        return signal.default_int_handler(sig, frame)
 
 
 original_signal_handler = signal.signal(signal.SIGINT, signal_handler)
@@ -59,10 +64,18 @@ class Client(object):
                  versions.
     :param request_timeout: A timeout in seconds for requests. The default is
                             5 seconds.
+    :param ssl_verify: ``True`` to verify SSL certificates, or ``False`` to
+                       skip SSL certificate verification, allowing
+                       connections to servers with self signed certificates.
+                       The default is ``True``.
     """
     event_names = ['connect', 'disconnect', 'message']
 
-    def __init__(self, logger=False, json=None, request_timeout=5):
+    def __init__(self,
+                 logger=False,
+                 json=None,
+                 request_timeout=5,
+                 ssl_verify=True):
         self.handlers = {}
         self.base_url = None
         self.transports = None
@@ -77,9 +90,10 @@ class Client(object):
         self.read_loop_task = None
         self.write_loop_task = None
         self.ping_loop_task = None
-        self.ping_loop_event = self.create_event()
+        self.ping_loop_event = None
         self.queue = None
         self.state = 'disconnected'
+        self.ssl_verify = ssl_verify
 
         if json is not None:
             packet.Packet.json = json
@@ -274,7 +288,7 @@ class Client(object):
             self._reset()
             raise exceptions.ConnectionError(
                 'Connection refused by the server')
-        if r.status_code != 200:
+        if r.status_code < 200 or r.status_code >= 300:
             raise exceptions.ConnectionError(
                 'Unexpected status code {} in server response'.format(
                     r.status_code))
@@ -340,11 +354,17 @@ class Client(object):
         if self.http:
             cookies = '; '.join(["{}={}".format(cookie.name, cookie.value)
                                  for cookie in self.http.cookies])
+
         try:
-            ws = websocket.create_connection(
-                websocket_url + self._get_url_timestamp(), header=headers,
-                cookie=cookies)
-        except ConnectionError:
+            if not self.ssl_verify:
+                ws = websocket.create_connection(
+                    websocket_url + self._get_url_timestamp(), header=headers,
+                    cookie=cookies, sslopt={"cert_reqs": ssl.CERT_NONE})
+            else:
+                ws = websocket.create_connection(
+                    websocket_url + self._get_url_timestamp(), header=headers,
+                    cookie=cookies)
+        except (ConnectionError, IOError):
             if upgrade:
                 self.logger.warning(
                     'WebSocket upgrade failed: connection error')
@@ -448,7 +468,7 @@ class Client(object):
             self.http = requests.Session()
         try:
             return self.http.request(method, url, headers=headers, data=body,
-                                     timeout=timeout)
+                                     timeout=timeout, verify=self.ssl_verify)
         except requests.exceptions.RequestException as exc:
             self.logger.info('HTTP %s request to %s failed with error %s.',
                              method, url, exc)
@@ -495,7 +515,8 @@ class Client(object):
         interval.
         """
         self.pong_received = True
-        self.ping_loop_event.clear()
+        if self.ping_loop_event is None:
+            self.ping_loop_event = self.create_event()
         while self.state == 'connected':
             if not self.pong_received:
                 self.logger.info(
@@ -522,7 +543,7 @@ class Client(object):
                     'Connection refused by the server, aborting')
                 self.queue.put(None)
                 break
-            if r.status_code != 200:
+            if r.status_code < 200 or r.status_code >= 300:
                 self.logger.warning('Unexpected status code %s in server '
                                     'response, aborting', r.status_code)
                 self.queue.put(None)
@@ -540,7 +561,8 @@ class Client(object):
         self.logger.info('Waiting for write loop task to end')
         self.write_loop_task.join()
         self.logger.info('Waiting for ping loop task to end')
-        self.ping_loop_event.set()
+        if self.ping_loop_event:  # pragma: no cover
+            self.ping_loop_event.set()
         self.ping_loop_task.join()
         if self.state == 'connected':
             self._trigger_event('disconnect', run_async=False)
@@ -575,7 +597,8 @@ class Client(object):
         self.logger.info('Waiting for write loop task to end')
         self.write_loop_task.join()
         self.logger.info('Waiting for ping loop task to end')
-        self.ping_loop_event.set()
+        if self.ping_loop_event:  # pragma: no cover
+            self.ping_loop_event.set()
         self.ping_loop_task.join()
         if self.state == 'connected':
             self._trigger_event('disconnect', run_async=False)
@@ -629,7 +652,7 @@ class Client(object):
                     self.logger.warning(
                         'Connection refused by the server, aborting')
                     break
-                if r.status_code != 200:
+                if r.status_code < 200 or r.status_code >= 300:
                     self.logger.warning('Unexpected status code %s in server '
                                         'response, aborting', r.status_code)
                     self._reset()
